/////////////////////////////////////////
// Simple Oscilloscope                 //
// by Norman Liebich <norm@d-parc.be>  //
/////////////////////////////////////////

import processing.opengl.*;
import pitaru.sonia_v2_9.*;


Sample sample_0;
Sample sample_1;

float[] buffer;
float[] empty_data;
int y_center = 240,
 y_fac = y_center/2,
 current_buffer = 0,
 screenshot_num=0,
 max_frames = 1102,    // also app.width
 new_max_frames = max_frames,
 max_width = max_frames,
 top=y_center-y_fac,
 bottom=y_center+y_fac;

int sample_frequency= 44100;
int sample_duration= (max_frames/sample_frequency)*1000;
float time_unit= float(1);
boolean init=true,
  init_buffer=true,
  reinit=false,
  is_pause=false;

void setup () {
  size(max_frames, y_center*2, OPENGL);
  Sonia.start(this); 
  LiveInput.start();
  
  PFont feisar = loadFont("Feisar-12.vlw");
  textFont(feisar, 12);
  
  empty_data= new float[0];
  initSamples();
  stroke(200);
  fill(230, 190, 190);
}

void initSamples() {
  buffer= new float[max_frames];
  sample_0 = new Sample(max_frames, sample_frequency);
  sample_1 = new Sample(max_frames, sample_frequency);
  sample_duration= (max_frames/sample_frequency)*1000;
  float t = float(max_frames) / float(sample_frequency);
  time_unit= t * 250.0;
}

void getSoundBuffer() {
  if (current_buffer==0) {
    if (init_buffer) {
      LiveInput.startRec(sample_0, 0, max_frames);
      delay(sample_duration);
      LiveInput.startRec(sample_1, 0, max_frames);
      sample_0.read(buffer);
      init_buffer=false;
    }
    else {
      delay(((max_frames - sample_0.getNumFrames()) / sample_frequency )*1000);
      LiveInput.startRec(sample_1, 0, max_frames);
      sample_0.read(buffer);
      current_buffer=1;
    }
  }
  else {
    delay(((max_frames - sample_1.getNumFrames()) / sample_frequency )*1000);
    LiveInput.startRec(sample_0, 0, max_frames);
    sample_1.read(buffer);
    current_buffer=0;
  }
}
void draw() {
  background(0);
  if (reinit) {
    if (current_buffer==0) {
      LiveInput.stopRec(sample_0);
    }
    else {
      LiveInput.stopRec(sample_1);
    }
    delay(sample_duration);
    init_buffer=true;
    current_buffer=0;
    max_frames=new_max_frames;
    initSamples();
    reinit=false;
    println("max_frames="+max_frames);
  }
  if (init) {
    init=false;
  }
  else {
    stroke(120,255,120);
    if (!is_pause) {
      getSoundBuffer();
    }
    int add= max_frames/max_width;
    add = add <= 0 ? 1 : add;
    int zz=0;
    for (int z=0; z< max_width-1; z++) {
      line(z, y_center- int(buffer[zz]*y_fac), 2, z+1, y_center-int(buffer[zz+add]*y_fac), 2);
      zz += add;
    }
  }
  drawGrid();
}

void drawGrid() {
  stroke(100);
  line(0, y_center, 0, max_frames, y_center,0);
  stroke(70);
  line(0, top, 0, max_frames, top, 0); 
  line(0, bottom, 0, max_frames, bottom, 0);
  line(1, top, 2, 1, bottom,2);
  line(220, top, 2, 220, bottom,2);
  line(441, top, 2, 441, bottom,2);
  line(661, top, 2, 661, bottom,2);
  line(882, top, 2, 882, bottom,2);
  line(1102, top, 2, 1102, bottom,2);
  
  text( time_unit, 200, top-2);
  text( time_unit*2, 420, top-2);
  text( time_unit*3, 640, top-2);
  text( time_unit*4, 862, top-2);
  text( time_unit*5, 1050, top-2);
  
}

void keyPressed() {
  switch (keyCode) {
    case UP:
      new_max_frames -= max_frames==max_width ? 0 : max_width;
      reinit=true;
      break;
    case DOWN:
      new_max_frames += max_frames==max_width*15 ? 0 : max_width;
      reinit=true;
      break;
    case ENTER:
      saveFrame();
      break;
    case 32:
      // SPACE
      is_pause= is_pause ? false : true;
      break;
    default:
      println(keyCode);
  } 
}

public void stop(){ 
  Sonia.stop(); 
  super.stop(); 
} 

